/*
	This file is part of the smb library
    Copyright (C) 1999  Nicolas Brodu
    brodu@iie.cnam.fr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
// SMBio manages the connections to a remote host
#ifndef __NMBIO_H__
#define __NMBIO_H__

#include "order.h"

// Define our own hostent class, mainly for a clean destuctor
// (if a <netdb.h> struct hostent was used, it would be up to
// the caller to destroy each member separately)
class NBHostEnt
{
public:
	char *NBName; // NetBIOS name
	char *name;   // clear asciiz name
	uint32 ip;
	NBHostEnt *next; // for group names
	NBHostEnt(const char *nbn=0, const char *n=0, uint32 i=0, NBHostEnt *ne=0);
	~NBHostEnt();
};

class NBHostCache
{
public:
	NBHostCache();
	~NBHostCache();
	uint32 lastCheck, timeout;
	NBHostEnt *host; // with a 0 ip if was not found
	NBHostCache *next;
};

class NMBIO
{
protected:
	char *decodeNBName(const char* NBName);
	char *ourName;
	char *ourNBName;
	NBHostCache *cache;
	uint32 netaddr; // guess ?
	struct sockaddr_in *socknetaddr; // parameters of the connection
public:
	NMBIO(const char *us=0); // can specify our host name
	~NMBIO();
	int errno;
	// return a string containing ourName, which will be deallocated by caller
	char *getOurName();
	// return a string containing ourNBName, which will be deallocated by caller
	char *getOurNBName();
	// Works with DNS or NetBIOS names. In case of a conflict
	// between NetBIOS and DNS, NetBIOS is used.
	// At present, IP dot notation is not supported, and
	// In case of a group name, only the members IP are valid
	// In the future, the list of the member names will be
	// returned as well (see NBHostEnt class)
	struct NBHostEnt *gethostbyname(const char *name); //,int flag=0);
	// doesn't work !
	struct NBHostEnt *gethostbyaddr(uint32 IP);
};


#endif //__NMBIO_H__
